@@ -76,7 +76,7 @@ class SpiderController(Thread):
                     continue
 
                 # 将 requests callback 还原
-                elif isinstance(task, Request) and task.callback and hasattr(self.spider, task.callback):
+                elif isinstance(task, Request):
                     task.callback = getattr(self.spider, task.callback)
 
                 self.waiting = False
@@ -125,7 +125,7 @@ class SpiderController(Thread):
             new_request.url = response.urljoin(new_request.url)
 
         # callback 转化为字符串
-        if new_request.callback:
+        if new_request.callback and not isinstance(new_request.callback, str):
             new_request.callback = new_request.callback.__name__  # 转成字符串，不然无法序列化
 
         # 添加请求必须参数
@@ -164,7 +164,7 @@ class SpiderController(Thread):
                 for middleware in self.__class__.REQUEST_MIDDLEWARE:
                     new_request = middleware.request_error(self.spider, request, e)
                     if new_request is None:
-                        pass
+                        continue
                     elif isinstance(new_request, Request):
                         self.add_new_request(new_request=new_request, old_request=request, response=response)
                         raise DropRequestException(middleware.__name__, request)
@@ -181,7 +181,7 @@ class SpiderController(Thread):
         for middleware in self.__class__.REQUEST_MIDDLEWARE:
             new_request = middleware.request_close(self.spider, request, response)
             if new_request is None:
-                pass
+                continue
             elif isinstance(new_request, Request):
                 self.add_new_request(new_request=new_request, old_request=request, response=response)
                 raise DropRequestException(middleware.__name__, request)
