@@ -1,6 +1,7 @@
 """
     请求的处理
 """
+import json
 import time
 import random
 import urllib3
@@ -48,7 +49,7 @@ class Request:
         'callback',
         'cookie_jar',
         'priority',
-        'command'
+        'command',
     ]
 
     # 下载器
@@ -240,12 +241,62 @@ class Request:
         request_dict = {}
 
         for key, value in self.__dict__.items():
-            if key.startswith('_') or not value:
+            # 无 callback 要添加
+            if key == 'callback' and value is None:
+                request_dict[key] = 'parse'
+
+            # _打头的名字 和 无值的忽略
+            elif key.startswith('_') or not value:
                 continue
-            request_dict[key] = value
+
+            # priority 默认值的情况直接忽略
+            elif key == 'priority' and value == settings.DEFAULT_QUEUE_PRIORITY:
+                continue
+
+            # cookie_jar 转为字典
+            elif key == 'cookie_jar':
+                request_dict[key] = value.get_dict()
+
+            # downloader 为默认的，直接忽略
+            elif key == 'downloader' and value == self.__class__.DOWNLOADER:
+                continue
+
+            # downloader 转换为模块
+            elif key == 'downloader':
+                request_dict[key] = {
+                    'module': value.__module__,
+                    'init': value.__name__,
+                }
+
+            # downloader_parser 为默认的，直接忽略
+            elif key == 'downloader_parser' and value == self.__class__.DOWNLOADER_PARSER:
+                continue
+
+            # downloader_parser 转换为模块
+            elif key == 'downloader_parser':
+                request_dict[key] = {
+                    'module': value.__module__,
+                    'init': value.__name__,
+                }
+
+            # 其余直接赋值
+            else:
+                request_dict[key] = value
 
         return request_dict
 
+    def to_json(self, **kwargs) -> json:
+        """
+        转化为 json 字符串
+
+        :param kwargs: json 参数
+        :return:
+        """
+
+        kwargs.setdefault('ensure_ascii', False)
+
+        return json.dumps(self.to_dict(), **kwargs)
+
     @property
     def domain(self) -> str:
         """
@@ -291,3 +342,59 @@ class Request:
 
     def __str__(self):
         return f"<Request {self.method}-{self.url}>"
+
+
+class LoadRequest:
+    """
+        从字典加载出 请求
+    """
+    CACHE = {}
+
+    @classmethod
+    def load_from_dict(cls, **kwargs) -> Request:
+        """
+        从字典导入
+
+        :param kwargs: request 参数
+        """
+        for key, value in kwargs.items():
+            # 导入 cookie_jar
+            if key == 'cookie_jar':
+                kwargs[key] = RequestsCookieJar()
+                kwargs[key].update(value)
+
+            # 导入 downloader
+            elif key == 'downloader':
+                kwargs[key] = cls._load_module(value)
+
+            # 导入 downloader_parser
+            elif key == 'downloader_parser':
+                kwargs[key] = cls._load_module(value)
+
+        return Request(**kwargs)
+
+    @classmethod
+    def load_from_json(cls, data: str) -> Request:
+        """
+        从 json 字符串导入
+
+        :return:
+        """
+
+        return cls.load_from_dict(**json.loads(data))
+
+    @classmethod
+    def _load_module(cls, value):
+        """
+        导入模块
+
+        :param value: 导入模块的路径
+        """
+        module_path = value['module'] + '.' + value['init']
+        if module_path not in cls.CACHE:
+            module = import_module(module_path, instantiate=False)[0]  # 导入
+            cls.CACHE[module_path] = module
+        else:
+            module = cls.CACHE[module_path]
+
+        return module
